% The VARs were run on several different data sets. In each case,
% to estimate the equilibrium model you need the point estimates
% of the impulse response functions, as well as their variance covariance
% matrix (the diagonal part is needed for estimation of the model,
% and the full matrix is used in the computation of the standard errors of the
% model coefficients.) These are the 'reduced form' representations of the
% data on which estimation was done.

% Four different reduced forms were used in the analysis, based on analysis
% of different data sets and VARs. One way to reproduce these reduced forms
% is to run main.m with estvar = imp = 1. This will automatically produce
% the reduced form that you want (i.e., the one that corresponds to how you
% set the switches at the start of main.m). You can then proceed to do 
% model analysis on that reduced form. The program handles everything
% automatically.
% However, creating a reduced form is slightly time-consuming, depending on
% your setting of ndraws. If you set it at 1,000 it takes a while to do all
% the simulations that are involved in creating the reduced form.
% By running this program you have access to pre-packaged reduced forms,
% and you can skip the step of running estvar = imp = 1.

% You just have to set lld, and execute this code before you run main.m
% 

lld=1;%1 benchmark model, VAR(4)
      %2 VAR(6) version of benchmark model
      %3 use private sector business hours with VAR(4)
      %4 use business sector productivity data

%$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$      
% The commands which load in the data are: 
% load var20pi_yunits
% load compare20pi_yunits
% load VAR_IRFsAndSEs
% 
% For the benchmark model, these file names are stored with the name 'bench'
% afterward.
% 
% For the files with settings,
% product == 2 & criterion == 0 & FR == 0 & sm == 0 & period == 1 & money == 1 & pinvestment == 1 & hours == 0
% They *mat files are stored with the name businessprod afterward


%this is the code for storing the VAR(6) results....
% clear all
% close all
% load var20pi_yunits
% save var20pi_yunitsnlags6
% clear
% load compare20pi_yunits
% save compare20pi_yunitsnlags6
% clear
% load VAR_IRFsAndSEs
% save VAR_IRFsAndSEsnlags6

%this is the code for recovering the VAR(6) results....
if lld == 2
    clear all
    close all
    load var20pi_yunitsnlags6
    save var20pi_yunits
    clear
    load compare20pi_yunitsnlags6
    save compare20pi_yunits
    clear
    load VAR_IRFsAndSEsnlags6
    save VAR_IRFsAndSEs
    return
end
%this is the code for storing the businessprod data....
% clear all
% close all
% load var20pi_yunits
% save var20pi_yunitsbusinessprod
% clear
% load compare20pi_yunits
% save compare20pi_yunitsbusinessprod
% clear
% load VAR_IRFsAndSEs
% save VAR_IRFsAndSEsbusinessprod

%this is the code for recovering the businessprod data....
if lld == 4
    clear all
    close all
    load var20pi_yunitsbusinessprod
    save var20pi_yunits
    clear
    load compare20pi_yunitsbusinessprod
    save compare20pi_yunits
    clear
    load VAR_IRFsAndSEsbusinessprod
    save VAR_IRFsAndSEs
    return
end


%this is the code for recovering the benchmark data.....
if lld == 1
    clear all
    close all
    load var20pi_yunitsbench
    save var20pi_yunits
    clear
    load compare20pi_yunitsbench
    save compare20pi_yunits
    clear
    load VAR_IRFsAndSEsbench
    save VAR_IRFsAndSEs
    return
end

%business hours data:
%product == 0 & criterion == 0 & FR == 0 & sm == 0 & period == 1 & money == 1 & pinvestment == 1 & hours == 1
%this is the code for storing the businesshours data....
% clear all
% close all
% load var20pi_yunits
% save var20pi_yunitsbusinesshours
% clear
% load compare20pi_yunits
% save compare20pi_yunitsbusinesshours
% clear
% load VAR_IRFsAndSEs
% save VAR_IRFsAndSEsbusinesshours

%this is the code for recovering the businesshours data....
if lld == 3
    clear all
    close all
    load var20pi_yunitsbusinesshours
    save var20pi_yunits
    clear
    load compare20pi_yunitsbusinesshours
    save compare20pi_yunits
    clear
    load VAR_IRFsAndSEsbusinesshours
    save VAR_IRFsAndSEs
    return
end